<?php
namespace FreePBX\modules;

class Qxact_reports implements \BMO {
	public function __construct($freepbx = null) {
		if ($freepbx == null) {
			throw new Exception("Not given a FreePBX Object");
		}
		$this->FreePBX = $freepbx;
		$this->db = $freepbx->Database;
	}
  public function install() {}
  public function uninstall() {}
  public function backup() {}
  public function restore($backup) {}
  public function doConfigPageInit($page) {
    $action = isset($_REQUEST['action'])?$_REQUEST['action']:'';
    switch ($action) {
      case 'addtemplate':
        $id = $this->addTemplate($_REQUEST['template_name'], $_REQUEST);
        $_REQUEST['id'] = $id;
      break;
      case 'edittemplate':
        $this->updateTemplate($_REQUEST['id'], $_REQUEST['template_name'], $_REQUEST);
      break;
      case 'delete':
        $this->delTemplate($_REQUEST['id']);
      break;
    }
  }
	public function chownFreepbx() {
		$moduledir = __DIR__;
		$files = array();
		$files[] = array('type' => 'file',
			'path' => $moduledir.'/import_queue_data.php',
			'perms' => 0755);
		return $files;
	}
	public function getActionBar($request) {
		$buttons = array();
		switch($request['display']) {
			case 'qxact_reports':
        $buttons = array(
          'submit' => array(
            'name' => 'submit',
            'id' => 'submit',
            'value' => _("Get Q-Xact Report")
          )
        );
      break;
			case 'qxact_admin':
				$buttons = array(
          'addtable' => array(
            'name' => 'addtable',
            'id' => 'addtable',
            'value' => 'addTable'
          ),
					'delete' => array(
						'name' => 'delete',
						'id' => 'delete',
						'value' => _('Delete')
					),
					'reset' => array(
						'name' => 'reset',
						'id' => 'reset',
						'value' => _('Reset')
					),
					'submit' => array(
						'name' => 'submit',
						'id' => 'submit',
						'value' => _('Submit')
					),
				);
				if (empty($request['id'])) {
					unset($buttons['delete']);
				}
				if(!isset($request['view'])){
					$buttons = array();
				}
			break;
		}
		return $buttons;
	}
  public function getTemplate($id){
    $sql = "SELECT * FROM qxact_reports WHERE id = ?";
    $stmt = $this->db->prepare($sql);
    $stmt->execute(array($id));
    $ret = $stmt->fetchALL(\PDO::FETCH_ASSOC);
    if($ret){
      $data = unserialize($ret[0]['data']);
      $name = $ret[0]['name'];
      return array('name' => $name, 'data' => $data);
    }
    return false;
  }
  public function addTemplate($name,$data){
    $data = serialize($data);
    $sql = 'INSERT INTO qxact_reports (name,data) VALUES (:template_name,:template_data)';
    $stmt = $this->db->prepare($sql);
    $ret = $stmt->execute(array('template_name'=> $name, 'template_data' => $data));
    if($ret){
      return $this->db->lastInsertId();
    }else{
      return $ret;
    }
  }
  public function updateTemplate($id, $name, $data){
    $data = serialize($data);
    $sql = 'UPDATE qxact_reports SET name = :template_name, data = :template_data WHERE id = :template_id';
    $stmt = $this->db->prepare($sql);
    $ret = $stmt->execute(array('template_id'=> $id, 'template_name'=> $name, 'template_data' => $data));
    return $ret;
  }
  public function delTemplate($id){
    $sql = "DELETE FROM qxact_reports WHERE id = ?";
    $stmt = $this->db->prepare($sql);
    $ret = $stmt->execute(array($id));
    return $ret;
  }
  public function listTemplates(){
    $sql = 'SELECT id, name FROM qxact_reports ORDER BY name ASC';
    $stmt = $this->db->prepare($sql);
    $stmt->execute();
    $ret = $stmt->fetchALL(\PDO::FETCH_ASSOC);
    if($ret){
      return $ret;
    }
    return array();
  }
	public function listQueues(){
		$sql = 'SELECT DISTINCT(queuename) FROM qxact_calls UNION SELECT DISTINCT(queuename) FROM qxact_agent_calls UNION SELECT DISTINCT(queuename) FROM qxact_agent_actions ORDER BY queuename';
		$stmt = $this->db->prepare($sql);
		$stmt->execute();
		$ret = $stmt->fetchALL(\PDO::FETCH_ASSOC);
		if($ret){
			return $ret;
		}
		return array();
	}
	public function listAgents(){
		$sql = "SELECT DISTINCT(agent) FROM qxact_calls where agent > '' UNION SELECT DISTINCT(agent) FROM qxact_agent_calls UNION SELECT DISTINCT(agent) FROM qxact_agent_actions ORDER BY agent";
		$stmt = $this->db->prepare($sql);
		$stmt->execute();
		$ret = $stmt->fetchALL(\PDO::FETCH_ASSOC);
		if($ret){
			return $ret;
		}
		return array();
	}
  public function ajaxRequest($req, &$setting) {
       switch ($req) {
           case 'getJSON':
               return true;
           break;
           default:
               return false;
           break;
       }
   }
   public function ajaxHandler(){
       switch ($_REQUEST['command']) {
           case 'getJSON':
              switch ($_REQUEST['jdata']) {
                case 'templategrid':
                  return array_values($this->listTemplates());
                break;
              }
           break;

           default:
               return false;
           break;
       }
   }
	 public function getRightNav($request) {
	   if(isset($request['view']) && $request['view'] == 'form'){
	     return load_view(__DIR__."/views/bootnav.php",array());
	   }
	 }
	public function validateTimestamp($unixtimestamp){
		if($unixtimestamp >  PHP_INT_MAX){
			return false;
		}
		$currentTime = new \DateTime();
		//90 days in to the future... too many? to few?
		$futureTime = $currentTime->add(new \DateInterval('P90D'));
		$stamp = new \DateTime();
		$stamp->setTimestamp($unixtimestamp);
		if($stamp > $futureTime ){
			dbug("over 90");
			return false;
		}
		return true;
	 }
}
