//	License for all code of this FreePBX module can be found in the license file inside the module directory
//	Copyright 2013 Schmooze Com Inc.
//
/**
 * This is the master FreePBX Object used to control and operate FreePBX
 * @type {[type]}
 */
var EventEmitter = require( "events" ).EventEmitter,
		server = require("http").Server(),
		io = require("socket.io")(server),
		freepbx = {},
		obj = {},
		run = 0,
		port = 8001,
		host = "0.0.0.0",
		enabled = true;

Server = function(fpbx) {
	fpbx.server = this;

	var config = {},
			amistatus = "disconnected",
			context = {},
			properties = [ "on", "once", "addListener", "removeListener", "removeAllListeners",
											"listeners", "setMaxListeners", "emit" ];

	freepbx = fpbx;
	context.emitter = new EventEmitter();
	context.held = [];

	properties.map(function(property) {
		Object.defineProperty(obj, property, {
			value: context.emitter[property].bind(context.emitter)
		});
	});

	enabled = fpbx.config.get("NODEJSENABLED");
	host = fpbx.config.get("NODEJSBINDADDRESS") ? fpbx.config.get("NODEJSBINDADDRESS") : host;
	port = fpbx.config.get("NODEJSBINDPORT") ? fpbx.config.get("NODEJSBINDPORT") : port;

	io.use(checkAuth);
	start();
	amistatus = "connected";

	fpbx.astman.on("close", function() {
		if (amistatus != "disconnected") {
			fpbx.emit("disconnect");
			stop();
			amistatus = "disconnected";
		}
	});

	fpbx.astman.on("connect", function() {
		if (amistatus != "connected") {
			amistatus = "connected";
			start();
		}
	});

	obj.io = io;
	obj.stop = stop;
	obj.start = start;
	return obj;
};

start = function() {
	if (!server.address() && enabled) {
		console.log("Listening on port " + host + ":" + port);
		server.listen(port, host);
	} else if (!server.address() && !enabled) {
		console.log("Server is disabled. Exiting");
		process.exit(1);
	}
};

stop = function() {
	if (server.address()) {
		console.log("Shutting down server on port " + host + ":" + port);
		server.close();
	}
};

checkAuth = function(socket, next) {
	var auth = false,
			address = null,
			suppliedToken = (typeof socket.handshake.query.token != "undefined") ? socket.handshake.query.token : "empty";

	run++; // 0 -> 1
	suppliedToken = freepbx.db.escape(suppliedToken);
	address = freepbx.db.escape(socket.handshake.address);
	freepbx.db.query("SELECT * FROM ucp_sessions WHERE session = '" + suppliedToken + "' AND address = '" + address + "'").on("result", function(res) {
		res.on("row", function(row) {
			auth = true;
		})
		.on("end", function() {
			if (auth) {
				console.log("Token [" + suppliedToken + "] from: " + socket.handshake.address + " was accepted");
				next();
			} else {
				console.log("Token [" + suppliedToken + "] from: " + socket.handshake.address + " was rejected");
				next(new Error("not authorized"));
			}
		})
		.on("error", function(e) {
			console.log("Error while checking authorization?");
			next(new Error("not authorized"));
		});
	});
};

module.exports = Server;
